@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,10 +16,21 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/**
+ * @file
+ * Network listener threads
+ */
+
 #ifndef _H_LISTENER
 #define _H_LISTENER
 
-// Thread listening on SIP UDP port
+/** Thread listening on SIP UDP port */
 void *listen_udp(void *arg);
 
+/** Thread listening on established TCP connections */
+void *listen_for_data_tcp(void *arg);
+
+/** Thread listening for incoming TCP connection requests */
+void *listen_for_conn_requests_tcp(void *arg);
+
 #endif
