@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,9 +16,13 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <algorithm>
+
 #include "definitions.h"
 #include "hdr_privacy.h"
 
+using namespace std;
+
 t_hdr_privacy::t_hdr_privacy() : t_header("Privacy") {};
 
 void t_hdr_privacy::add_privacy(const string &privacy) {
