@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
+    Copyright (C) 2005-2008  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,9 +16,13 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <cstdlib>
+
 #include "media_type.h"
 #include "util.h"
 
+using namespace std;
+
 t_media::t_media() {
 	q = 1.0;
 }
@@ -68,7 +72,12 @@ string t_media::encode(void) const {
 		s += charset;
 	}
 	s += param_list2str(media_param_list);
-	if (q != 1) s += float2str(q, ";q=%.1f");
+	
+	if (q != 1) {
+		s += ";q=";
+		s += float2str(q, 1);
+	}
+	
 	s += param_list2str(accept_extension_list);
 
 	return s;
